@@ -4,15 +4,28 @@
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
 #ifndef ___VBox_shflsvc_h
@@ -265,6 +278,17 @@ typedef enum _SHFLCREATERESULT
 /** Do not allow access. */
 #define SHFL_CF_ACCESS_DENYALL          (SHFL_CF_ACCESS_DENYREAD | SHFL_CF_ACCESS_DENYWRITE)
 
+/** Requested access to attributes of the object. */
+#define SHFL_CF_ACCESS_MASK_ATTR        (0x00030000)
+
+/** No access requested. */
+#define SHFL_CF_ACCESS_ATTR_NONE        (0x00000000)
+/** Read access requested. */
+#define SHFL_CF_ACCESS_ATTR_READ        (0x00010000)
+/** Write access requested. */
+#define SHFL_CF_ACCESS_ATTR_WRITE       (0x00020000)
+/** Read/Write access requested. */
+#define SHFL_CF_ACCESS_ATTR_READWRITE   (SHFL_CF_ACCESS_READ | SHFL_CF_ACCESS_WRITE)
 
 /** @} */
 
@@ -320,13 +344,13 @@ typedef struct _SHFLDIRINFO
 {
     /** Full information about the object. */
     RTFSOBJINFO     Info;
-    /** The length of the short field (number of RTUCS2 chars).
+    /** The length of the short field (number of RTUTF16 chars).
      * It is 16-bit for reasons of alignment. */
     uint16_t        cucShortName;
     /** The short name for 8.3 compatability.
      * Empty string if not available.
      */
-    RTUCS2          uszShortName[14];
+    RTUTF16         uszShortName[14];
     /** @todo malc, a description, please. */
     SHFLSTRING      name;
 } SHFLDIRINFO, *PSHFLDIRINFO;
@@ -433,7 +457,7 @@ typedef struct _VBoxSFMapFolder_Old
      */
     HGCMFunctionParameter root;
 
-    /** pointer, in: RTUCS2
+    /** pointer, in: RTUTF16
      * Path delimiter
      */
     HGCMFunctionParameter delimiter;
@@ -462,7 +486,7 @@ typedef struct _VBoxSFMapFolder
      */
     HGCMFunctionParameter root;
 
-    /** pointer, in: RTUCS2
+    /** pointer, in: RTUTF16
      * Path delimiter
      */
     HGCMFunctionParameter delimiter;
@@ -471,7 +495,7 @@ typedef struct _VBoxSFMapFolder
      * Case senstive flag
      */
     HGCMFunctionParameter fCaseSensitive;
-    
+
 } VBoxSFMapFolder;
 
 /** Number of parameters */
@@ -916,63 +940,24 @@ typedef struct _VBoxSFRename
 
 /**
  * SHFL_FN_ADD_MAPPING
+ * Host call, no guest structure is used.
  */
 
-/** Parameters structure. */
-typedef struct _VBoxSFAddMapping
-{
-    VBoxGuestHGCMCallInfo callInfo;
-
-    /** pointer, in: Folder name
-     * Points to SHFLSTRING buffer.
-     */
-    HGCMFunctionParameter folder;
-
-    /** pointer, in: Mapping name
-     * Points to SHFLSTRING buffer.
-     */
-    HGCMFunctionParameter mapping;
-
-} VBoxSFAddMapping;
-
-#define SHFL_CPARMS_ADD_MAPPING  (2)
-
+#define SHFL_CPARMS_ADD_MAPPING  (3)
 
 /**
  * SHFL_FN_REMOVE_MAPPING
+ * Host call, no guest structure is used.
  */
 
-/** Parameters structure. */
-typedef struct _VBoxSFRemoveMapping
-{
-    VBoxGuestHGCMCallInfo callInfo;
-
-    /** pointer, in: Guest name
-     * Points to SHFLSTRING buffer.
-     */
-    HGCMFunctionParameter path;
-
-} VBoxSFRemoveMapping;
-
 #define SHFL_CPARMS_REMOVE_MAPPING (1)
 
 
 /**
  * SHFL_FN_SET_STATUS_LED
+ * Host call, no guest structure is used.
  */
 
-/** Parameters structure. */
-typedef struct _VBoxSFSetStatusLed
-{
-    VBoxGuestHGCMCallInfo callInfo;
-
-    /** pointer, in: LED address
-     * Points to PPDMLED buffer.
-     */
-    HGCMFunctionParameter led;
-
-} VBoxSFSetStatusLed;
-
 #define SHFL_CPARMS_SET_STATUS_LED (1)
 
 /** @} */
