@@ -1,17 +1,30 @@
 /** @file
- * innotek Portable Runtime - File I/O.
+ * IPRT - File I/O.
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
 #ifndef ___iprt_file_h
@@ -84,6 +97,11 @@ __BEGIN_DECLS
 /** Action mask. */
 #define RTFILE_O_ACTION_MASK        0x00000300
 
+/** Turns off indexing of files on Windows hosts, *CREATE* only.
+ * @remark  This might not be implemented on all platforms,
+ *          and will be ignored on those.
+ */
+#define RTFILE_O_NOT_CONTENT_INDEXED 0x00000800
 /** Truncate the file.
  * @remark  This will not truncate files opened for read-only.
  * @remark  The trunction doesn't have to be atomically, so anyone
@@ -104,10 +122,34 @@ __BEGIN_DECLS
  */
 #define RTFILE_O_WRITE_THROUGH      0x00008000
 
+/** File attributes access, *CREATE* only.
+ * @remark  This might not be implemented on all platforms,
+ *          and will be ignored on those.
+ */
+/** Attributes can be read if the file is being opened
+ * with read access, and can be written with write access.
+ */
+#define RTFILE_O_ACCESS_ATTR_DEFAULT 0x00000000
+/** Attributes can be read. */
+#define RTFILE_O_ACCESS_ATTR_READ   0x00010000
+/** Attributes can be written. */
+#define RTFILE_O_ACCESS_ATTR_WRITE  0x00020000
+/** Attributes can be both read & written. */
+#define RTFILE_O_ACCESS_ATTR_READWRITE 0x00030000
+/** The file attributes access mask. */
+#define RTFILE_O_ACCESS_ATTR_MASK   0x00030000
+
+/** Unix file mode mask for use when creating files. */
+#define RTFILE_O_CREATE_MODE_MASK   0x1ff00000
+/** The number of bits to shift to get the file mode mask.
+ * To extract it: (fFlags & RTFILE_O_CREATE_MODE_MASK) >> RTFILE_O_CREATE_MODE_SHIFT.
+ */
+#define RTFILE_O_CREATE_MODE_SHIFT  20
+
 /** Mask of all valid flags.
  * @remark  This doesn't validate the access mode properly.
  */
-#define RTFILE_O_VALID_MASK         0x0000F333
+#define RTFILE_O_VALID_MASK          0x1ff3FB73
 
 /** @} */
 
@@ -141,6 +183,23 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
  */
 RTR3DECL(int)  RTFileClose(RTFILE File);
 
+/**
+ * Creates an IPRT file handle from a native one.
+ *
+ * @returns IPRT status code.
+ * @param   pFile           Where to store the IPRT file handle.
+ * @param   uNative         The native handle.
+ */
+RTR3DECL(int) RTFileFromNative(PRTFILE pFile, RTHCINTPTR uNative);
+
+/**
+ * Gets the native handle for an IPRT file handle.
+ *
+ * @return  The native handle.
+ * @params  File            The IPRT file handle.
+ */
+RTR3DECL(RTHCINTPTR) RTFileToNative(RTFILE File);
+
 /**
  * Delete a file.
  *
@@ -255,6 +314,35 @@ RTR3DECL(int)  RTFileSetSize(RTFILE File, uint64_t cbSize);
  */
 RTR3DECL(int)  RTFileGetSize(RTFILE File, uint64_t *pcbSize);
 
+/**
+ * Determine the maximum file size.
+ *
+ * @returns The max size of the file.
+ *          -1 on failure, the file position is undefined.
+ * @param   File        Handle to the file.
+ * @see     RTFileGetMaxSizeEx.
+ */
+RTR3DECL(RTFOFF) RTFileGetMaxSize(RTFILE File);
+
+/**
+ * Determine the maximum file size.
+ *
+ * @returns IPRT status code.
+ * @param   File        Handle to the file.
+ * @param   pcbMax      Where to store the max file size.
+ * @see     RTFileGetMaxSize.
+ */
+RTR3DECL(int) RTFileGetMaxSizeEx(RTFILE File, PRTFOFF pcbMax);
+
+/**
+ * Determine the maximum file size depending on the file system the file is stored on.
+ *
+ * @returns The max size of the file.
+ *          -1 on failure.
+ * @param   File        Handle to the file.
+ */
+RTR3DECL(RTFOFF) RTFileGetMaxSize(RTFILE File);
+
 /**
  * Gets the current file position.
  *
@@ -297,6 +385,18 @@ RTDECL(int) RTFileCopy(const char *pszSrc, const char *pszDst);
  */
 RTDECL(int) RTFileCopyByHandles(RTFILE FileSrc, RTFILE FileDst);
 
+/** Flags for RTFileCopyEx().
+ * @{ */
+/** Do not use RTFILE_O_DENY_WRITE on the source file to allow for copying files opened for writing. */
+#define RTFILECOPY_FLAGS_NO_SRC_DENY_WRITE  RT_BIT(0)
+/** Do not use RTFILE_O_DENY_WRITE on the target file. */
+#define RTFILECOPY_FLAGS_NO_DST_DENY_WRITE  RT_BIT(1)
+/** Do not use RTFILE_O_DENY_WRITE on either of the two files. */
+#define RTFILECOPY_FLAGS_NO_DENY_WRITE      ( RTFILECOPY_FLAGS_NO_SRC_DENY_WRITE | RTFILECOPY_FLAGS_NO_DST_DENY_WRITE )
+/** */
+#define RTFILECOPY_FLAGS_MASK               UINT32_C(0x00000003)
+/** @} */
+
 /**
  * Copies a file.
  *
@@ -306,10 +406,11 @@ RTDECL(int) RTFileCopyByHandles(RTFILE FileSrc, RTFILE FileDst);
  * @param   pszSrc      The path to the source file.
  * @param   pszDst      The path to the destination file.
  *                      This file will be created.
+ * @param   fFlags      Flags (RTFILECOPY_*).
  * @param   pfnProgress Pointer to callback function for reporting progress.
  * @param   pvUser      User argument to pass to pfnProgress along with the completion precentage.
  */
-RTDECL(int) RTFileCopyEx(const char *pszSrc, const char *pszDst, PFNRTPROGRESS pfnProgress, void *pvUser);
+RTDECL(int) RTFileCopyEx(const char *pszSrc, const char *pszDst, uint32_t fFlags, PFNRTPROGRESS pfnProgress, void *pvUser);
 
 /**
  * Copies a file given the handles to both files and
@@ -388,7 +489,7 @@ RTDECL(int) RTFileMove(const char *pszSrc, const char *pszDst, unsigned fMove);
  * operation system. Also see comments to RTFileChangeLock API call.
  *
  * Linux/Posix: By default locks in Unixes are advisory. This means that cooperating processes
- * may use locks to coordonate access to a file between themselves, but programs are also free
+ * may use locks to coordinate access to a file between themselves, but programs are also free
  * to ignore locks and access the file in any way they choose to.
  *
  * Additional reading:
@@ -580,6 +681,84 @@ RTR3DECL(int) RTFileGetOwner(RTFILE File, uint32_t *pUid, uint32_t *pGid);
  */
 RTR3DECL(int) RTFileIoCtl(RTFILE File, int iRequest, void *pvData, unsigned cbData, int *piRet);
 
+/**
+ * Reads the file into memory.
+ *
+ * The caller must free the memory using RTFileReadAllFree().
+ *
+ * @returns IPRT status code.
+ * @param   pszFilename     The name of the file.
+ * @param   ppvFile         Where to store the pointer to the memory on successful return.
+ * @param   pcbFile         Where to store the size of the file on successful return.
+ *
+ * @remarks Note that this function may be implemented using memory mapping, which means
+ *          that the file may remain open until RTFileReadAllFree() is called. It also
+ *          means that the return memory may reflect the state of the file when it's
+ *          accessed instead of when this call was done. So, in short, don't use this
+ *          API for volatile files, then rather use the extended variant with a
+ *          yet-to-be-defined.
+ */
+RTDECL(int) RTFileReadAll(const char *pszFilename, void **ppvFile, size_t *pcbFile);
+
+/**
+ * Reads the file into memory.
+ *
+ * The caller must free the memory using RTFileReadAllFree().
+ *
+ * @returns IPRT status code.
+ * @param   pszFilename     The name of the file.
+ * @param   off             The offset to start reading at.
+ * @param   cbMax           The maximum number of bytes to read into memory. Specify RTFOFF_MAX
+ *                          to read to the end of the file.
+ * @param   fFlags          Flags for the future, must be 0.
+ * @param   ppvFile         Where to store the pointer to the memory on successful return.
+ * @param   pcbFile         Where to store the size of the file on successful return.
+ *
+ * @remarks See the remarks for RTFileReadAll.
+ */
+RTDECL(int) RTFileReadAllEx(const char *pszFilename, RTFOFF off, RTFOFF cbMax, uint32_t fFlags, void **ppvFile, size_t *pcbFile);
+
+/**
+ * Reads the file into memory.
+ *
+ * The caller must free the memory using RTFileReadAllFree().
+ *
+ * @returns IPRT status code.
+ * @param   File            The handle to the file.
+ * @param   ppvFile         Where to store the pointer to the memory on successful return.
+ * @param   pcbFile         Where to store the size of the file on successful return.
+ *
+ * @remarks See the remarks for RTFileReadAll.
+ */
+RTDECL(int) RTFileReadAllByHandle(RTFILE File, void **ppvFile, size_t *pcbFile);
+
+/**
+ * Reads the file into memory.
+ *
+ * The caller must free the memory using RTFileReadAllFree().
+ *
+ * @returns IPRT status code.
+ * @param   File            The handle to the file.
+ * @param   off             The offset to start reading at.
+ * @param   cbMax           The maximum number of bytes to read into memory. Specify RTFOFF_MAX
+ *                          to read to the end of the file.
+ * @param   fFlags          Flags for the future, must be 0.
+ * @param   ppvFile         Where to store the pointer to the memory on successful return.
+ * @param   pcbFile         Where to store the size of the file on successful return.
+ *
+ * @remarks See the remarks for RTFileReadAll.
+ */
+RTDECL(int) RTFileReadAllByHandleEx(RTFILE File, RTFOFF off, RTFOFF cbMax, uint32_t fFlags, void **ppvFile, size_t *pcbFile);
+
+/**
+ * Frees the memory returned by one of the RTFileReadAll(), RTFileReadAllEx(),
+ * RTFileReadAllByHandle() and RTFileReadAllByHandleEx() functions.
+ *
+ * @param   pvFile          Pointer to the memory.
+ * @param   cbFile          The size of the memory.
+ */
+RTDECL(void) RTFileReadAllFree(void *pvFile, size_t cbFile);
+
 #endif /* IN_RING3 */
 
 /** @} */
