@@ -4,15 +4,19 @@
  *
  * Display driver screen draw entry points.
  *
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
 /* The driver operates in 3 modes:
@@ -82,8 +86,8 @@ BOOL bIsScreenSurface (SURFOBJ *pso)
     if (pso)
     {
         PPDEV ppdev = (PPDEV)pso->dhpdev;
-        
-        /* The screen surface has the 'pso->dhpdev' field, 
+
+        /* The screen surface has the 'pso->dhpdev' field,
          * and is either the screen device surface with handle = hsurfScreen,
          * or a surface derived from DDRAW with address equal to the framebuffer.
          */
@@ -100,6 +104,7 @@ BOOL bIsScreenSurface (SURFOBJ *pso)
     return FALSE;
 }
 
+#ifndef VBOX_WITH_HGSMI
 #define VBVA_OPERATION(__psoDest, __fn, __a) do {                     \
     if (bIsScreenSurface(__psoDest))                                  \
     {                                                                 \
@@ -128,6 +133,36 @@ BOOL bIsScreenSurface (SURFOBJ *pso)
         }                                                             \
     }                                                                 \
 } while (0)
+#else
+#define VBVA_OPERATION(__psoDest, __fn, __a) do {                      \
+    if (bIsScreenSurface(__psoDest))                                   \
+    {                                                                  \
+        PPDEV ppdev = (PPDEV)__psoDest->dhpdev;                        \
+                                                                       \
+        if (ppdev->bHGSMISupported && vboxHwBufferBeginUpdate (ppdev)) \
+        {                                                              \
+            vbva##__fn __a;                                            \
+                                                                       \
+            if (  ppdev->pVBVA->u32HostEvents                           \
+                & VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET)            \
+            {                                                          \
+                vrdpReset (ppdev);                                     \
+                                                                       \
+                ppdev->pVBVA->u32HostEvents &=                          \
+                          ~VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET;   \
+            }                                                          \
+                                                                       \
+            if (ppdev->pVBVA->u32HostEvents                             \
+                & VBVA_F_MODE_VRDP)                                    \
+            {                                                          \
+                vrdp##__fn __a;                                        \
+            }                                                          \
+                                                                       \
+            vboxHwBufferEndUpdate (ppdev);                             \
+        }                                                              \
+    }                                                                  \
+} while (0)
+#endif /* VBOX_WITH_HGSMI */
 
 //#undef VBVA_OPERATION
 //#define VBVA_OPERATION(_psoDest, __fn, __a) do { } while (0)
@@ -297,9 +332,9 @@ BOOL APIENTRY DrvCopyBits(
     dumpsurf(psoDest, "psoDest");
 
     STATPRINT;
-    
+
 #ifdef VBOX_VBVA_ADJUST_RECT
-    /* Experimental fix for too large bitmap updates. 
+    /* Experimental fix for too large bitmap updates.
      *
      * Some application do a large bitmap update event if only
      * a small part of the bitmap is actually changed.
@@ -307,7 +342,7 @@ BOOL APIENTRY DrvCopyBits(
      * The driver will find the changed rectangle by comparing
      * the current framebuffer content with the source bitmap.
      *
-     * The optimization is only active when: 
+     * The optimization is only active when:
      *  - the VBVA extension is enabled;
      *  - the source bitmap is not cacheable;
      *  - the bitmap formats of both the source and the screen surfaces are equal.
@@ -319,12 +354,19 @@ BOOL APIENTRY DrvCopyBits(
     {
         PPDEV ppdev = (PPDEV)psoDest->dhpdev;
 
+#ifndef VBOX_WITH_HGSMI
         VBVAMEMORY *pVbvaMemory = ppdev->vbva.pVbvaMemory;
 
         DISPDBG((1, "offscreen->screen\n"));
 
         if (   pVbvaMemory
             && (pVbvaMemory->fu32ModeFlags & VBVA_F_MODE_ENABLED))
+#else
+        DISPDBG((1, "offscreen->screen\n"));
+
+        if (   ppdev->pVBVA
+            && (ppdev->pVBVA->u32HostEvents & VBVA_F_MODE_ENABLED))
+#endif /* VBOX_WITH_HGSMI */
         {
             if (   (psoSrc->fjBitmap & BMF_DONTCACHE) != 0
                 || psoSrc->iUniq == 0)
@@ -429,6 +471,34 @@ BOOL APIENTRY DrvStrokePath(
     return bRc;
 }
 
+BOOL APIENTRY DrvStrokeAndFillPath(
+    SURFOBJ   *pso,
+    PATHOBJ   *ppo,
+    CLIPOBJ   *pco,
+    XFORMOBJ  *pxo,
+    BRUSHOBJ  *pboStroke,
+    LINEATTRS *plineattrs,
+    BRUSHOBJ  *pboFill,
+    POINTL    *pptlBrushOrg,
+    MIX        mixFill,
+    FLONG      flOptions
+    )
+{
+    BOOL bRc;
+
+    DISPDBG((1, "%s\n", __FUNCTION__));
+
+    STATDRVENTRY(StrokeAndFillPath, pso);
+
+    bRc = EngStrokeAndFillPath(CONV_SURF(pso), ppo, pco, pxo, pboStroke, plineattrs, pboFill, pptlBrushOrg, mixFill, flOptions);
+
+    VBVA_OPERATION(pso,
+                   StrokeAndFillPath,
+                   (pso, ppo, pco, pxo, pboStroke, plineattrs, pboFill, pptlBrushOrg, mixFill, flOptions));
+
+    return bRc;
+}
+
 static void ssbDiscardTopSlot (PPDEV ppdev)
 {
     SSB *pSSB = &ppdev->aSSB[--ppdev->cSSB];
@@ -592,7 +662,7 @@ ULONG_PTR APIENTRY DrvSaveScreenBits(
         {
             DISPDBG((1, "DrvSaveScreenBits: SS_SAVE %d\n", ppdev->cSSB));
 
-            if (ppdev->cSSB >= ELEMENTS(ppdev->aSSB))
+            if (ppdev->cSSB >= RT_ELEMENTS(ppdev->aSSB))
             {
                 /* All slots are already in use. Fail. */
                 DISPDBG((1, "DrvSaveScreenBits: no more slots %d!!!\n", ppdev->cSSB));
@@ -701,6 +771,7 @@ BOOL APIENTRY DrvRealizeBrush(
     {
         PPDEV ppdev = (PPDEV)psoTarget->dhpdev;
 
+#ifndef VBOX_WITH_HGSMI
         if (   ppdev->vbva.pVbvaMemory
             && (ppdev->vbva.pVbvaMemory->fu32ModeFlags & VBVA_F_MODE_ENABLED))
         {
@@ -719,6 +790,26 @@ BOOL APIENTRY DrvRealizeBrush(
                 bRc = vrdpRealizeBrush (pbo, psoTarget, psoPattern, psoMask, pxlo, iHatch);
             }
         }
+#else
+        if (   ppdev->pVBVA
+            && (ppdev->pVBVA->u32HostEvents & VBVA_F_MODE_ENABLED))
+        {
+            if (ppdev->pVBVA->u32HostEvents
+                & VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET)
+            {
+                vrdpReset (ppdev);
+
+                ppdev->pVBVA->u32HostEvents &=
+                    ~VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET;
+            }
+
+            if (ppdev->pVBVA->u32HostEvents
+                & VBVA_F_MODE_VRDP)
+            {
+                bRc = vrdpRealizeBrush (pbo, psoTarget, psoPattern, psoMask, pxlo, iHatch);
+            }
+        }
+#endif /* VBOX_WITH_HGSMI */
     }
 
     return bRc;
