@@ -1,4 +1,4 @@
-/** $Id: clipboard.cpp 18360 2009-03-26 23:46:38Z vboxsync $ */
+/** $Id: clipboard.cpp $ */
 /** @file
  * Guest Additions - X11 Shared Clipboard.
  */
@@ -389,6 +389,14 @@ static void vboxClipboardGetCText(XtPointer pValue, size_t cbSourceLen)
     LogFlowFunc(("\n"));
     LogFlow(("vboxClipboardGetCText: converting compound text to Utf-16LE. Original is %.*s\n",
            cbSourceLen, pValue));
+    /* Quick fix for 2.2. */
+    if (cbSourceLen == 0)
+    {
+        XtFree(reinterpret_cast<char *>(pValue));
+        vboxClipboardSendData(VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT,
+                              NULL, 0);
+        return;
+    }
     /* First convert the compound text to Utf8 */
     property.value = reinterpret_cast<unsigned char *>(pValue);
     property.encoding = g_ctx.atomCText;
