@@ -1,4 +1,4 @@
-/* $Id: dnsproxy.c 18815 2009-04-07 12:34:27Z vboxsync $ */
+/* $Id: dnsproxy.c $ */
 /*
  * Copyright (c) 2003,2004,2005 Armin Wolfermann
  *
@@ -105,7 +105,7 @@ timeout(PNATState pData, struct socket *so, void *arg)
 {
     struct request *req = (struct request *)arg;
     struct dns_entry *de;
-    de = LIST_NEXT(req->dns_server, de_list);
+    de = TAILQ_PREV(req->dns_server, dns_list_head, de_list);
     /* here we should check if we reached the end of the DNS server list */
     if (de == NULL)
     {
@@ -264,10 +264,21 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
         req->id = QUERYID;
         memcpy(&req->client, &fromaddr, sizeof(struct sockaddr_in));
         memcpy(&req->clientid, &buf[0], 2);
+        req->dns_server = TAILQ_LAST(&pData->dns_list_head, dns_list_head);
+        if (req->dns_server == NULL)
+        {
+            static int fail_counter = 0;
+            RTMemFree(req);
+            if (fail_counter == 0)
+                LogRel(("NAT/dnsproxy: Empty DNS entry (suppressed 100 times)\n"));
+            else 
+                fail_counter = (fail_counter == 100 ? 0 : fail_counter++);
+            return;
+            
+        }
+        retransmit = 0;
         so->so_timeout = timeout;
         so->so_timeout_arg = req;
-        retransmit = 0;
-        req->dns_server = LIST_FIRST(&pData->dns_list_head);
         req->nbyte = byte;
         memcpy(req->byte, buf, byte); /* copying original request */
     } 
