@@ -379,6 +379,11 @@ static int vhdDynamicHeaderUpdate(PVHDIMAGE pImage)
     ddh.Checksum = 0;
     ddh.Checksum = RT_H2BE_U32(vhdChecksum(&ddh, sizeof(ddh)));
     rc = RTFileWriteAt(pImage->File, pImage->u64DataOffset, &ddh, sizeof(ddh), NULL);
+    if (RT_FAILURE(rc))
+        return rc;
+
+    /* Update the VHD footer copy. */
+    rc = RTFileWriteAt(pImage->File, 0, &pImage->vhdFooterCopy, sizeof(VHDFooter), NULL);
 
 out:
     return rc;
@@ -1185,7 +1190,11 @@ static int vhdSetUuid(void *pBackendData, PCRTUUID pUuid)
     if (pImage)
     {
         pImage->ImageUuid = *pUuid;
-        /**@todo: implement */
+        /* Update the footer copy. It will get written to disk when the image is closed. */
+        memcpy(&pImage->vhdFooterCopy.UniqueID, pUuid, 16);
+        /* Update checksum. */
+        pImage->vhdFooterCopy.Checksum = 0;
+        pImage->vhdFooterCopy.Checksum = RT_H2BE_U32(vhdChecksum(&pImage->vhdFooterCopy, sizeof(VHDFooter)));
         rc = VINF_SUCCESS;
     }
     else
