@@ -5,24 +5,30 @@
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2009 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_GUI
+#define __STDC_LIMIT_MACROS
+#define __STDC_CONSTANT_MACROS
+
 #ifdef RT_OS_DARWIN
 # include <Carbon/Carbon.h>
-# define OSType VBoxOSType
 # undef PAGE_SIZE
 # undef PAGE_SHIFT
 #endif
@@ -39,7 +45,7 @@
 #include <VBox/log.h>
 #include <iprt/path.h>
 #include <iprt/string.h>
-#include <iprt/runtime.h>
+#include <iprt/initterm.h>
 #include <iprt/assert.h>
 #include <iprt/semaphore.h>
 #include <iprt/stream.h>
@@ -48,7 +54,9 @@
 
 #ifdef VBOXBFE_WITH_X11
 # include <X11/Xlib.h>
-# include <X11/Xcursor/Xcursor.h>
+# ifndef VBOXBFE_WITHOUT_XCURSOR
+#  include <X11/Xcursor/Xcursor.h>
+# endif
 #endif
 
 #include "VBoxBFE.h"
@@ -192,7 +200,7 @@ CONEVENT SDLConsole::eventWait()
                             enmHKeyState = HKEYSTATE_USED;
                             break;
                         }
-                        if (VBOX_SUCCESS(rc))
+                        if (RT_SUCCESS(rc))
                         {
                             return CONEVENT_QUIT;
                         }
@@ -231,7 +239,7 @@ CONEVENT SDLConsole::eventWait()
                     if (ev->type == SDL_KEYDOWN)
                     {
                         int rc = handleHostKey(&ev->key);
-                        if (VBOX_SUCCESS(rc) && rc != VINF_SUCCESS)
+                        if (RT_SUCCESS(rc) && rc != VINF_SUCCESS)
                         {
                             return CONEVENT_QUIT;
                         }
@@ -432,7 +440,7 @@ void SDLConsole::eventQuit()
     doEventQuit();
 }
 
-#if defined(RT_OS_DARWIN)
+#if defined(RT_OS_DARWIN) || defined(RT_OS_SOLARIS) || defined(RT_OS_OS2)
 /**
  * Fallback keycode conversion using SDL symbols.
  *
@@ -607,8 +615,13 @@ uint8_t SDLConsole::keyEventToKeyCode(const SDL_KeyboardEvent *ev)
     // start with the scancode determined by SDL
     keycode = ev->keysym.scancode;
 
-#ifdef VBOXBFE_WITH_X11 /// @todo verify that these are X11 issues and not unique linux issues.
-    // workaround for SDL keyboard translation issues on X11
+#if defined(RT_OS_LINUX)
+    // workaround for SDL keyboard translation issues on Linux
+    // keycodes > 0x100 are sent as 0xe0 keycode
+    // Note that these are the keycodes used by XFree86/X.org
+    // servers on a Linux host, and will almost certainly not
+    // work on other hosts or on other servers on Linux hosts.
+    // For a more general approach, see the Wine code in the GUI.
     static const uint8_t x_keycode_to_pc_keycode[61] =
     {
        0xc7,      /*  97  Home   */
@@ -866,6 +879,11 @@ uint8_t SDLConsole::keyEventToKeyCode(const SDL_KeyboardEvent *ev)
     RTPrintf("scancode=%#x -> %#x\n", ev->keysym.scancode, keycode);
 #endif
 
+#elif defined(RT_OS_SOLARIS) || defined(RT_OS_OS2)
+    /*
+     * For now, just use the fallback code.
+     */
+    keycode = Keyevent2KeycodeFallback(ev);
 #endif
     return keycode;
 }
@@ -1250,7 +1268,7 @@ void SDLConsole::updateTitlebar()
     char pszTitle[1024];
 
     RTStrPrintf(pszTitle, sizeof(pszTitle),
-                "innotek VirtualBox%s%s",
+                "Sun VirtualBox%s%s",
                 g_uProgressPercent == ~0U && machineState == VMSTATE_SUSPENDED ? " - [Paused]" : "",
                 mfInputGrab                                                    ? " - [Input captured]": "");
 
@@ -1282,7 +1300,7 @@ void SDLConsole::updateTitlebar()
     }
 #endif /* DEBUG */
 
-    SDL_WM_SetCaption(pszTitle, "innotek VirtualBox");
+    SDL_WM_SetCaption(pszTitle, "Sun vXM VirtualBox");
 }
 
 /**
@@ -1293,8 +1311,8 @@ void SDLConsole::updateTitlebarProgress(const char *pszStr, int iPercent)
 {
     char szTitle[256];
     AssertMsg(iPercent >= 0 && iPercent <= 100, ("%d\n", iPercent));
-    RTStrPrintf(szTitle, sizeof(szTitle), "innotek VirtualBox - %s %d%%...", pszStr, iPercent);
-    SDL_WM_SetCaption(szTitle, "innotek VirtualBox");
+    RTStrPrintf(szTitle, sizeof(szTitle), "Sun VirtualBox - %s %d%%...", pszStr, iPercent);
+    SDL_WM_SetCaption(szTitle, "Sun VirtualBox");
 }
 
 /**
@@ -1419,7 +1437,7 @@ void SDLConsole::setPointerShape (const PointerShapeChangeData *data)
         if (hBitmap)
             ::DeleteObject (hBitmap);
 
-#elif defined(VBOXBFE_WITH_X11)
+#elif defined(VBOXBFE_WITH_X11) && !defined(VBOXBFE_WITHOUT_XCURSOR)
 
         XcursorImage *img = XcursorImageCreate (data->width, data->height);
         Assert (img);
@@ -1589,7 +1607,7 @@ int SDLConsole::handleHostKey(const SDL_KeyboardEvent *pEv)
          */
         case SDLK_h:
         {
-            VMCtrlACPIButton();
+            VMCtrlACPIPowerButton();
             break;
         }
 
@@ -1612,3 +1630,5 @@ int SDLConsole::handleHostKey(const SDL_KeyboardEvent *pEv)
 
     return VINF_SUCCESS;
 }
+
+
