@@ -1,4 +1,4 @@
-/* $Id: VBoxManageHelp.cpp 18809 2009-04-07 11:57:07Z vboxsync $ */
+/* $Id: VBoxManageHelp.cpp $ */
 /** @file
  * VBoxManage - help and other message output.
  */
@@ -179,6 +179,7 @@ void printUsage(USAGECATEGORY u64Cmd)
                  "                            [--uartmode<1-N> disconnected|\n"
                  "                                             server <pipe>|\n"
                  "                                             client <pipe>|\n"
+                 "                                             file <file>|\n"
                  "                                             <devicename>]\n"
 #ifdef VBOX_WITH_MEM_BALLOONING
                  "                            [--guestmemoryballoon <balloonsize in MB>]\n"
@@ -344,9 +345,7 @@ void printUsage(USAGECATEGORY u64Cmd)
         RTPrintf("VBoxManage modifyhd         <uuid>|<filename>\n"
                  "                            [--type normal|writethrough|immutable]\n"
                  "                            [--autoreset on|off]\n"
-#if 0
                  "                            [--compact]\n"
-#endif
                  "\n");
     }
 
