@@ -1,4 +1,4 @@
-/* $Id: VBoxManageInfo.cpp 18829 2009-04-07 15:42:32Z vboxsync $ */
+/* $Id: VBoxManageInfo.cpp $ */
 /** @file
  * VBoxManage - The 'showvminfo' command and helper routines.
  */
@@ -840,6 +840,14 @@ HRESULT showVMInfo (ComPtr<IVirtualBox> virtualBox,
                         else
                             RTPrintf(", disconnected\n");
                         break;
+                    case PortMode_RawFile:
+                        if (details == VMINFO_MACHINEREADABLE)
+                            RTPrintf("uartmode%d=\"%lS\"\n", currentUART + 1,
+                                     path.raw());
+                        else
+                            RTPrintf(", attached to raw file '%lS'\n",
+                                     path.raw());
+                        break;
                     case PortMode_HostPipe:
                         if (details == VMINFO_MACHINEREADABLE)
                             RTPrintf("uartmode%d=\"%s,%lS\"\n", currentUART + 1,
