@@ -1,4 +1,4 @@
-/* $Id: VBoxManageModifyVM.cpp 18829 2009-04-07 15:42:32Z vboxsync $ */
+/* $Id: VBoxManageModifyVM.cpp $ */
 /** @file
  * VBoxManage - Implementation of modifyvm command.
  */
@@ -669,6 +669,11 @@ int handleModifyVM(HandlerArg *a)
                         return errorArgument("Uart pipe must start with \\\\.\\pipe\\");
 #endif
                 }
+                else if (!strcmp(a->argv[i], "file"))
+                {
+                    uarts_mode[n - 1] = a->argv[i];
+                    i++;
+                }
                 else
                 {
                     uarts_mode[n - 1] = (char*)"device";
@@ -1276,15 +1281,15 @@ int handleModifyVM(HandlerArg *a)
             ComPtr<IStorageController> storageController;
             CHECK_ERROR(machine, GetStorageControllerByName(Bstr("IDE"), storageController.asOutParam()));
 
-            if (RTStrICmp(idecontroller, "PIIX3"))
+            if (!RTStrICmp(idecontroller, "PIIX3"))
             {
                 CHECK_ERROR(storageController, COMSETTER(ControllerType)(StorageControllerType_PIIX3));
             }
-            else if (RTStrICmp(idecontroller, "PIIX4"))
+            else if (!RTStrICmp(idecontroller, "PIIX4"))
             {
                 CHECK_ERROR(storageController, COMSETTER(ControllerType)(StorageControllerType_PIIX4));
             }
-            else if (RTStrICmp(idecontroller, "ICH6"))
+            else if (!RTStrICmp(idecontroller, "ICH6"))
             {
                 CHECK_ERROR(storageController, COMSETTER(ControllerType)(StorageControllerType_ICH6));
             }
@@ -1717,6 +1722,10 @@ int handleModifyVM(HandlerArg *a)
                         CHECK_ERROR_RET(uart, COMSETTER(HostMode) (PortMode_HostPipe), 1);
                         CHECK_ERROR_RET(uart, COMSETTER(Server) (FALSE), 1);
                     }
+                    else if (!strcmp(uarts_mode[n], "file"))
+                    {
+                        CHECK_ERROR_RET(uart, COMSETTER(HostMode) (PortMode_RawFile), 1);
+                    }
                     else
                     {
                         CHECK_ERROR_RET(uart, COMSETTER(HostMode) (PortMode_HostDevice), 1);
