@@ -326,7 +326,7 @@ bool VBoxProblemReporter::askForOverridingFilesIfExists (const QStringList& aPat
 void VBoxProblemReporter::cannotDeleteFile (const QString& path, QWidget *aParent /* = NULL */) const
 {
     message (aParent, Error,
-             tr ("Failed to remove the file <b>%1</b>.<br /><br />Please try to remove that file yourself & try again.")
+             tr ("Failed to remove the file <b>%1</b>.<br /><br />Please try to remove the file yourself and try again.")
              .arg (path));
 }
 
@@ -747,7 +747,7 @@ void VBoxProblemReporter::cannotSendACPIToMachine()
             "does not use the ACPI subsystem."));
 }
 
-bool VBoxProblemReporter::warnAboutVirtNotEnabled()
+bool VBoxProblemReporter::warnAboutVirtNotEnabled64BitsGuest()
 {
     return messageOkCancel (mainWindowShown(), Error,
         tr ("<p>VT-x/AMD-V hardware acceleration has been enabled, but is "
@@ -759,6 +759,18 @@ bool VBoxProblemReporter::warnAboutVirtNotEnabled()
         tr ("Close VM"), tr ("Continue"));
 }
 
+bool VBoxProblemReporter::warnAboutVirtNotEnabledGuestRequired()
+{
+    return messageOkCancel (mainWindowShown(), Error,
+        tr ("<p>VT-x/AMD-V hardware acceleration has been enabled, but is "
+            "not operational. Certain guests (e.g. OS/2 and QNX) require "
+            "this feature.</p><p>Please ensure "
+            "that you have enabled VT-x/AMD-V properly in the BIOS of your "
+            "host computer.</p>"),
+        0 /* aAutoConfirmId */,
+        tr ("Close VM"), tr ("Continue"));
+}
+
 void VBoxProblemReporter::cannotSetSnapshotFolder (const CMachine &aMachine,
                                                    const QString &aPath)
 {
@@ -1225,13 +1237,14 @@ int VBoxProblemReporter::confirmDeletingHostInterface (const QString &aName,
                                                        QWidget *aParent)
 {
     return vboxProblem().message (aParent, VBoxProblemReporter::Question,
-        tr ("<p>Do you want to remove the selected host network interface "
-            "<nobr><b>%1</b>?</nobr></p>"
-            "<p><b>Note:</b> This interface may be in use by one or more "
-            "network adapters of this or another VM. After it is removed, these "
-            "adapters will no longer work until you correct their settings by "
-            "either choosing a different interface name or a different adapter "
-            "attachment type.</p>").arg (aName),
+        tr ("<p>Deleting this host-only network will lead to the deleting of "
+            "the host-only interface this network is based on. Do you want to "
+            "remove the (host-only network) interface <nobr><b>%1</b>?</nobr></p>"
+            "<p><b>Note:</b> this interface may be in use by one or more "
+            "virtual network adapters belonging to one of your VMs. "
+            "After it is removed, these adapters will no longer be usable until "
+            "you correct their settings by either choosing a different interface "
+            "name or a different adapter attachment type.</p>").arg (aName),
         0, /* autoConfirmId */
         QIMessageBox::Ok | QIMessageBox::Default,
         QIMessageBox::Cancel | QIMessageBox::Escape);
