@@ -39,6 +39,11 @@
 #undef LogFlow
 #define LogFlow Log
 
+/**
+ * @todo find a better solution for supporting the execute bit for non-windows
+ * guests on windows host. Search for "0111" to find all the relevant places.
+ */
+
 void vbsfStripLastComponent (char *pszFullPath, uint32_t cbFullPathRoot)
 {
     RTUNICP cp;
@@ -172,6 +177,69 @@ end:
     return rc;
 }
 
+static int vbsfPathCheck(const char *pUtf8Path, size_t cbPath)
+{
+    int rc = VINF_SUCCESS;
+
+    /* The pUtf8Path is what the guest sent. Verify that the path is within the root.
+     * Count '..' and other path components and check that we do not go over the root.
+     */
+
+    size_t i = 0;
+    int cComponents = 0; /* How many normal path components. */
+    int cParentDirs = 0; /* How many '..' components. */
+
+    for (;;)
+    {
+        /* Skip leading path delimiters. */
+        while (   i < cbPath
+               && (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'))
+            i++;
+
+        if (i >= cbPath)
+            break;
+
+        /* Check if that is a dot component. */
+        int cDots = 0;
+        while (i < cbPath && pUtf8Path[i] == '.')
+        {
+            cDots++;
+            i++;
+        }
+
+        if (   cDots >= 2 /* Consider all multidots sequences as a 'parent dir'. */
+            && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
+        {
+            cParentDirs++;
+        }
+        else if (   cDots == 1
+                 && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
+        {
+            /* Single dot, nothing changes. */
+        }
+        else
+        {
+            /* Skip this component. */
+            while (   i < cbPath
+                   && (pUtf8Path[i] != '\\' && pUtf8Path[i] != '/'))
+                i++;
+
+            cComponents++;
+        }
+
+        Assert(i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'));
+
+        /* Verify counters for every component. */
+        if (cParentDirs > cComponents)
+        {
+            rc = VERR_INVALID_NAME;
+            break;
+        }
+    }
+
+    return rc;
+}
+
 static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath,
                               uint32_t cbPath, char **ppszFullPath, uint32_t *pcbFullPathRoot, bool fWildCard = false)
 {
@@ -194,7 +262,14 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
         int rc;
         char *utf8Root;
 
-        rc = RTUtf16ToUtf8 (pwszRoot, &utf8Root);
+        /* Verify that the path is under the root directory. */
+        rc = vbsfPathCheck((char *)&pPath->String.utf8[0], pPath->u16Length);
+
+        if (RT_SUCCESS (rc))
+        {
+            rc = RTUtf16ToUtf8 (pwszRoot, &utf8Root);
+        }
+
         if (RT_SUCCESS (rc))
         {
             size_t cbUtf8Root, cbUtf8FullPath;
@@ -339,14 +414,26 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
                 {
                     AssertFailed();
 #ifdef RT_OS_DARWIN
-                	RTMemFree(pPath);
-                	pPath = pPathParameter;
+                    RTMemFree(pPath);
+                    pPath = pPathParameter;
 #endif
                     return rc;
                 }
 
                 uint32_t l = (uint32_t)strlen (dst);
 
+                /* Verify that the path is under the root directory. */
+                rc = vbsfPathCheck(dst, l);
+
+                if (RT_FAILURE(rc))
+                {
+#ifdef RT_OS_DARWIN
+                    RTMemFree(pPath);
+                    pPath = pPathParameter;
+#endif
+                    return rc;
+                }
+
                 cb -= l;
                 dst += l;
 
@@ -767,6 +854,9 @@ static int vbsfOpenFile (const char *pszPath, SHFLCREATEPARMS *pParms)
             /** @todo Possible race left here. */
             if (RT_SUCCESS(RTPathQueryInfo (pszPath, &info, RTFSOBJATTRADD_NOTHING)))
             {
+#ifdef RT_OS_WINDOWS
+                info.Attr.fMode |= 0111;
+#endif
                 pParms->Info = info;
             }
             pParms->Result = SHFL_FILE_EXISTS;
@@ -840,6 +930,9 @@ static int vbsfOpenFile (const char *pszPath, SHFLCREATEPARMS *pParms)
         rc = RTFileQueryInfo (pHandle->file.Handle, &info, RTFSOBJATTRADD_NOTHING);
         if (RT_SUCCESS(rc))
         {
+#ifdef RT_OS_WINDOWS
+            info.Attr.fMode |= 0111;
+#endif
             pParms->Info = info;
         }
     }
@@ -1037,6 +1130,9 @@ static int vbsfLookupFile(char *pszPath, SHFLCREATEPARMS *pParms)
     {
         case VINF_SUCCESS:
         {
+#ifdef RT_OS_WINDOWS
+            info.Attr.fMode |= 0111;
+#endif
             pParms->Info = info;
             pParms->Result = SHFL_FILE_EXISTS;
             break;
@@ -1423,6 +1519,9 @@ int vbsfDirList(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle, SHFLS
             return VINF_SUCCESS;    /* Return directly and don't free pDirEntry */
         }
 
+#ifdef RT_OS_WINDOWS
+        pDirEntry->Info.Attr.fMode |= 0111;
+#endif
         pSFDEntry->Info = pDirEntry->Info;
         pSFDEntry->cucShortName = 0;
 
@@ -1530,6 +1629,10 @@ int vbsfQueryFileInfo(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle,
     else
     {
         rc = RTFileQueryInfo(pHandle->file.Handle, pObjInfo, RTFSOBJATTRADD_NOTHING);
+#ifdef RT_OS_WINDOWS
+        if (RT_SUCCESS(rc) && RTFS_IS_FILE(pObjInfo->Attr.fMode))
+            pObjInfo->Attr.fMode |= 0111;
+#endif
     }
     if (rc == VINF_SUCCESS)
     {
@@ -1652,6 +1755,9 @@ static int vbsfSetEndOfFile(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE H
         rc = RTFileQueryInfo(pHandle->file.Handle, &fileinfo, RTFSOBJATTRADD_NOTHING);
         if (rc == VINF_SUCCESS)
         {
+#ifdef RT_OS_WINDOWS
+            fileinfo.Attr.fMode |= 0111;
+#endif
             *pSFDEntry = fileinfo;
             *pcbBuffer = sizeof(RTFSOBJINFO);
         }
