@@ -4,25 +4,31 @@
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
 #include "NetworkAdapterImpl.h"
 #include "Logging.h"
 #include "MachineImpl.h"
+#include "GuestOSTypeImpl.h"
 
 #include <iprt/string.h>
 #include <iprt/cpputils.h>
 
 #include <VBox/err.h>
+#include <VBox/settings.h>
 
 // constructor / destructor
 ////////////////////////////////////////////////////////////////////////////////
@@ -56,7 +62,7 @@ HRESULT NetworkAdapter::init (Machine *aParent, ULONG aSlot)
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_UNEXPECTED);
+    AssertReturn (autoInitSpan.isOk(), E_FAIL);
 
     unconst (mParent) = aParent;
     /* mPeer is left null */
@@ -67,7 +73,7 @@ HRESULT NetworkAdapter::init (Machine *aParent, ULONG aSlot)
     mData->mSlot = aSlot;
 
     /* default to Am79C973 */
-    mData->mAdapterType = NetworkAdapterType_NetworkAdapterAm79C973;
+    mData->mAdapterType = NetworkAdapterType_Am79C973;
 
     /* generate the MAC address early to guarantee it is the same both after
      * changing some other property (i.e. after mData.backup()) and after the
@@ -98,7 +104,7 @@ HRESULT NetworkAdapter::init (Machine *aParent, NetworkAdapter *aThat)
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_UNEXPECTED);
+    AssertReturn (autoInitSpan.isOk(), E_FAIL);
 
     unconst (mParent) = aParent;
     unconst (mPeer) = aThat;
@@ -106,7 +112,7 @@ HRESULT NetworkAdapter::init (Machine *aParent, NetworkAdapter *aThat)
     AutoCaller thatCaller (aThat);
     AssertComRCReturnRC (thatCaller.rc());
 
-    AutoReaderLock thatLock (aThat);
+    AutoReadLock thatLock (aThat);
     mData.share (aThat->mData);
 
     /* Confirm a successful initialization */
@@ -130,7 +136,7 @@ HRESULT NetworkAdapter::initCopy (Machine *aParent, NetworkAdapter *aThat)
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_UNEXPECTED);
+    AssertReturn (autoInitSpan.isOk(), E_FAIL);
 
     unconst (mParent) = aParent;
     /* mPeer is left null */
@@ -138,7 +144,7 @@ HRESULT NetworkAdapter::initCopy (Machine *aParent, NetworkAdapter *aThat)
     AutoCaller thatCaller (aThat);
     AssertComRCReturnRC (thatCaller.rc());
 
-    AutoReaderLock thatLock (aThat);
+    AutoReadLock thatLock (aThat);
     mData.attachCopy (aThat->mData);
 
     /* Confirm a successful initialization */
@@ -171,13 +177,12 @@ void NetworkAdapter::uninit()
 
 STDMETHODIMP NetworkAdapter::COMGETTER(AdapterType) (NetworkAdapterType_T *aAdapterType)
 {
-    if (!aAdapterType)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aAdapterType);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     *aAdapterType = mData->mAdapterType;
 
@@ -193,13 +198,18 @@ STDMETHODIMP NetworkAdapter::COMSETTER(AdapterType) (NetworkAdapterType_T aAdapt
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     /* make sure the value is allowed */
     switch (aAdapterType)
     {
-        case NetworkAdapterType_NetworkAdapterAm79C970A:
-        case NetworkAdapterType_NetworkAdapterAm79C973:
+        case NetworkAdapterType_Am79C970A:
+        case NetworkAdapterType_Am79C973:
+#ifdef VBOX_WITH_E1000
+        case NetworkAdapterType_I82540EM:
+        case NetworkAdapterType_I82543GC:
+        case NetworkAdapterType_I82545EM:
+#endif
             break;
         default:
             return setError (E_FAIL,
@@ -223,13 +233,12 @@ STDMETHODIMP NetworkAdapter::COMSETTER(AdapterType) (NetworkAdapterType_T aAdapt
 
 STDMETHODIMP NetworkAdapter::COMGETTER(Slot) (ULONG *aSlot)
 {
-    if (!aSlot)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aSlot);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     *aSlot = mData->mSlot;
 
@@ -238,13 +247,12 @@ STDMETHODIMP NetworkAdapter::COMGETTER(Slot) (ULONG *aSlot)
 
 STDMETHODIMP NetworkAdapter::COMGETTER(Enabled) (BOOL *aEnabled)
 {
-    if (!aEnabled)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aEnabled);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     *aEnabled = mData->mEnabled;
 
@@ -260,7 +268,7 @@ STDMETHODIMP NetworkAdapter::COMSETTER(Enabled) (BOOL aEnabled)
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     if (mData->mEnabled != aEnabled)
     {
@@ -278,13 +286,12 @@ STDMETHODIMP NetworkAdapter::COMSETTER(Enabled) (BOOL aEnabled)
 
 STDMETHODIMP NetworkAdapter::COMGETTER(MACAddress)(BSTR *aMACAddress)
 {
-    if (!aMACAddress)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aMACAddress);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     ComAssertRet (!!mData->mMACAddress, E_FAIL);
 
@@ -293,7 +300,7 @@ STDMETHODIMP NetworkAdapter::COMGETTER(MACAddress)(BSTR *aMACAddress)
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::COMSETTER(MACAddress)(INPTR BSTR aMACAddress)
+STDMETHODIMP NetworkAdapter::COMSETTER(MACAddress)(IN_BSTR aMACAddress)
 {
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
@@ -302,7 +309,7 @@ STDMETHODIMP NetworkAdapter::COMSETTER(MACAddress)(INPTR BSTR aMACAddress)
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     HRESULT rc = S_OK;
     bool emitChangeEvent = false;
@@ -376,13 +383,12 @@ STDMETHODIMP NetworkAdapter::COMSETTER(MACAddress)(INPTR BSTR aMACAddress)
 STDMETHODIMP NetworkAdapter::COMGETTER(AttachmentType)(
     NetworkAttachmentType_T *aAttachmentType)
 {
-    if (!aAttachmentType)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aAttachmentType);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     *aAttachmentType = mData->mAttachmentType;
 
@@ -391,44 +397,34 @@ STDMETHODIMP NetworkAdapter::COMGETTER(AttachmentType)(
 
 STDMETHODIMP NetworkAdapter::COMGETTER(HostInterface)(BSTR *aHostInterface)
 {
-    if (!aHostInterface)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aHostInterface);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     mData->mHostInterface.cloneTo (aHostInterface);
 
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::COMSETTER(HostInterface)(INPTR BSTR aHostInterface)
+STDMETHODIMP NetworkAdapter::COMSETTER(HostInterface)(IN_BSTR aHostInterface)
 {
     /** @todo Validate input string length. r=dmik: do it in XML schema?*/
 
-#ifdef RT_OS_WINDOWS
-    // we don't allow null strings for the host interface on Win32
-    // (because the @name attribute of <HostInerface> must be always present,
-    // but can be empty).
-    if (!aHostInterface)
-        return E_INVALIDARG;
-#endif
-#ifdef VBOX_WITH_UNIXY_TAP_NETWORKING
-    // empty strings are not allowed as path names
-    if (aHostInterface && !(*aHostInterface))
-        return E_INVALIDARG;
-#endif
+    /* we don't allow null strings for the host interface (because the @name
+     * attribute of <HostInterface> must be always present but can be empty). */
+    CheckComArgNotNull (aHostInterface);
 
-     AutoCaller autoCaller (this);
+    AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
     /* the machine needs to be mutable */
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     if (mData->mHostInterface != aHostInterface)
     {
@@ -444,176 +440,22 @@ STDMETHODIMP NetworkAdapter::COMSETTER(HostInterface)(INPTR BSTR aHostInterface)
     return S_OK;
 }
 
-#ifdef VBOX_WITH_UNIXY_TAP_NETWORKING
-
-STDMETHODIMP NetworkAdapter::COMGETTER(TAPFileDescriptor)(LONG *aTAPFileDescriptor)
-{
-    if (!aTAPFileDescriptor)
-        return E_POINTER;
-
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    AutoReaderLock alock (this);
-
-    *aTAPFileDescriptor = mData->mTAPFD;
-
-    return S_OK;
-}
-
-STDMETHODIMP NetworkAdapter::COMSETTER(TAPFileDescriptor)(LONG aTAPFileDescriptor)
-{
-    /*
-     * Validate input.
-     */
-    RTFILE tapFD = aTAPFileDescriptor;
-    if (tapFD != NIL_RTFILE && (LONG)tapFD != aTAPFileDescriptor)
-    {
-        AssertMsgFailed(("Invalid file descriptor: %ld.\n", aTAPFileDescriptor));
-        return setError (E_INVALIDARG,
-            tr ("Invalid file descriptor: %ld"), aTAPFileDescriptor);
-    }
-
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
-
-    AutoLock alock (this);
-
-    if (mData->mTAPFD != (RTFILE) aTAPFileDescriptor)
-    {
-        mData.backup();
-        mData->mTAPFD = aTAPFileDescriptor;
-
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        mParent->onNetworkAdapterChange (this);
-    }
-
-    return S_OK;
-}
-
-STDMETHODIMP NetworkAdapter::COMGETTER(TAPSetupApplication) (
-    BSTR *aTAPSetupApplication)
-{
-    if (!aTAPSetupApplication)
-        return E_POINTER;
-
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    AutoReaderLock alock (this);
-
-    /* we don't have to be in TAP mode to support this call */
-    mData->mTAPSetupApplication.cloneTo (aTAPSetupApplication);
-
-    return S_OK;
-}
-
-STDMETHODIMP NetworkAdapter::COMSETTER(TAPSetupApplication) (
-    INPTR BSTR aTAPSetupApplication)
-{
-    /* empty strings are not allowed as path names */
-    if (aTAPSetupApplication && !(*aTAPSetupApplication))
-        return E_INVALIDARG;
-
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
-
-    AutoLock alock (this);
-
-    if (mData->mTAPSetupApplication != aTAPSetupApplication)
-    {
-        mData.backup();
-        mData->mTAPSetupApplication = aTAPSetupApplication;
-
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        mParent->onNetworkAdapterChange (this);
-    }
-
-    return S_OK;
-}
-
-STDMETHODIMP NetworkAdapter::COMGETTER(TAPTerminateApplication) (
-    BSTR *aTAPTerminateApplication)
-{
-    if (!aTAPTerminateApplication)
-        return E_POINTER;
-
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    AutoReaderLock alock (this);
-
-    /* we don't have to be in TAP mode to support this call */
-    mData->mTAPTerminateApplication.cloneTo(aTAPTerminateApplication);
-
-    return S_OK;
-}
-
-STDMETHODIMP NetworkAdapter::COMSETTER(TAPTerminateApplication) (
-    INPTR BSTR aTAPTerminateApplication)
-{
-    /* empty strings are not allowed as path names */
-    if (aTAPTerminateApplication && !(*aTAPTerminateApplication))
-        return E_INVALIDARG;
-
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
-
-    AutoLock alock (this);
-
-    if (mData->mTAPTerminateApplication != aTAPTerminateApplication)
-    {
-        mData.backup();
-        mData->mTAPTerminateApplication = aTAPTerminateApplication;
-
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        mParent->onNetworkAdapterChange(this);
-    }
-
-    return S_OK;
-}
-
-#endif /* VBOX_WITH_UNIXY_TAP_NETWORKING */
-
 STDMETHODIMP NetworkAdapter::COMGETTER(InternalNetwork) (BSTR *aInternalNetwork)
 {
-    /* we don't allow null strings */
-    if (!aInternalNetwork)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aInternalNetwork);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     mData->mInternalNetwork.cloneTo (aInternalNetwork);
 
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::COMSETTER(InternalNetwork) (INPTR BSTR aInternalNetwork)
+STDMETHODIMP NetworkAdapter::COMSETTER(InternalNetwork) (IN_BSTR aInternalNetwork)
 {
-    if (!aInternalNetwork)
-        return E_INVALIDARG;
-
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
@@ -621,15 +463,17 @@ STDMETHODIMP NetworkAdapter::COMSETTER(InternalNetwork) (INPTR BSTR aInternalNet
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     if (mData->mInternalNetwork != aInternalNetwork)
     {
-        /* if an empty string is to be set, internal networking must be turned off */
-        if (   (aInternalNetwork == Bstr(""))
-            && (mData->mAttachmentType = NetworkAttachmentType_InternalNetworkAttachment))
+        /* if an empty/null string is to be set, internal networking must be
+         * turned off */
+        if (   (aInternalNetwork == NULL || *aInternalNetwork == '\0')
+            && mData->mAttachmentType == NetworkAttachmentType_Internal)
         {
-            return setError (E_FAIL, tr ("Empty internal network name is not valid"));
+            return setError (E_FAIL,
+                tr ("Empty or null internal network name is not valid"));
         }
 
         mData.backup();
@@ -644,15 +488,53 @@ STDMETHODIMP NetworkAdapter::COMSETTER(InternalNetwork) (INPTR BSTR aInternalNet
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::COMGETTER(CableConnected) (BOOL *aConnected)
+STDMETHODIMP NetworkAdapter::COMGETTER(NATNetwork) (BSTR *aNATNetwork)
 {
-    if (!aConnected)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aNATNetwork);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
+
+    mData->mNATNetwork.cloneTo (aNATNetwork);
+
+    return S_OK;
+}
+
+STDMETHODIMP NetworkAdapter::COMSETTER(NATNetwork) (IN_BSTR aNATNetwork)
+{
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    /* the machine needs to be mutable */
+    Machine::AutoMutableStateDependency adep (mParent);
+    CheckComRCReturnRC (adep.rc());
+
+    AutoWriteLock alock (this);
+
+    if (mData->mNATNetwork != aNATNetwork)
+    {
+        mData.backup();
+        mData->mNATNetwork = aNATNetwork;
+
+        /* leave the lock before informing callbacks */
+        alock.unlock();
+
+        mParent->onNetworkAdapterChange (this);
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP NetworkAdapter::COMGETTER(CableConnected) (BOOL *aConnected)
+{
+    CheckComArgOutPointerValid(aConnected);
+
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    AutoReadLock alock (this);
 
     *aConnected = mData->mCableConnected;
 
@@ -668,7 +550,7 @@ STDMETHODIMP NetworkAdapter::COMSETTER(CableConnected) (BOOL aConnected)
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     if (aConnected != mData->mCableConnected)
     {
@@ -684,15 +566,53 @@ STDMETHODIMP NetworkAdapter::COMSETTER(CableConnected) (BOOL aConnected)
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::COMGETTER(TraceEnabled) (BOOL *aEnabled)
+STDMETHODIMP NetworkAdapter::COMGETTER(LineSpeed) (ULONG *aSpeed)
 {
-    if (!aEnabled)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aSpeed);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
+
+    *aSpeed = mData->mLineSpeed;
+
+    return S_OK;
+}
+
+STDMETHODIMP NetworkAdapter::COMSETTER(LineSpeed) (ULONG aSpeed)
+{
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    /* the machine needs to be mutable */
+    Machine::AutoMutableStateDependency adep (mParent);
+    CheckComRCReturnRC (adep.rc());
+
+    AutoWriteLock alock (this);
+
+    if (aSpeed != mData->mLineSpeed)
+    {
+        mData.backup();
+        mData->mLineSpeed = aSpeed;
+
+        /* leave the lock before informing callbacks */
+        alock.unlock();
+
+        mParent->onNetworkAdapterChange (this);
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP NetworkAdapter::COMGETTER(TraceEnabled) (BOOL *aEnabled)
+{
+    CheckComArgOutPointerValid(aEnabled);
+
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    AutoReadLock alock (this);
 
     *aEnabled = mData->mTraceEnabled;
     return S_OK;
@@ -707,7 +627,7 @@ STDMETHODIMP NetworkAdapter::COMSETTER(TraceEnabled) (BOOL aEnabled)
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     if (aEnabled != mData->mTraceEnabled)
     {
@@ -725,20 +645,19 @@ STDMETHODIMP NetworkAdapter::COMSETTER(TraceEnabled) (BOOL aEnabled)
 
 STDMETHODIMP NetworkAdapter::COMGETTER(TraceFile) (BSTR *aTraceFile)
 {
-    if (!aTraceFile)
-        return E_POINTER;
+    CheckComArgOutPointerValid(aTraceFile);
 
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
-    AutoReaderLock alock (this);
+    AutoReadLock alock (this);
 
     mData->mTraceFile.cloneTo (aTraceFile);
 
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::COMSETTER(TraceFile) (INPTR BSTR aTraceFile)
+STDMETHODIMP NetworkAdapter::COMSETTER(TraceFile) (IN_BSTR aTraceFile)
 {
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
@@ -747,7 +666,7 @@ STDMETHODIMP NetworkAdapter::COMSETTER(TraceFile) (INPTR BSTR aTraceFile)
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     if (mData->mTraceFile != aTraceFile)
     {
@@ -775,15 +694,15 @@ STDMETHODIMP NetworkAdapter::AttachToNAT()
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
-    if (mData->mAttachmentType != NetworkAttachmentType_NATNetworkAttachment)
+    if (mData->mAttachmentType != NetworkAttachmentType_NAT)
     {
         mData.backup();
 
         detach();
 
-        mData->mAttachmentType = NetworkAttachmentType_NATNetworkAttachment;
+        mData->mAttachmentType = NetworkAttachmentType_NAT;
 
         /* leave the lock before informing callbacks */
         alock.unlock();
@@ -794,7 +713,7 @@ STDMETHODIMP NetworkAdapter::AttachToNAT()
     return S_OK;
 }
 
-STDMETHODIMP NetworkAdapter::AttachToHostInterface()
+STDMETHODIMP NetworkAdapter::AttachToBridgedInterface()
 {
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
@@ -803,17 +722,17 @@ STDMETHODIMP NetworkAdapter::AttachToHostInterface()
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     /* don't do anything if we're already host interface attached */
-    if (mData->mAttachmentType != NetworkAttachmentType_HostInterfaceNetworkAttachment)
+    if (mData->mAttachmentType != NetworkAttachmentType_Bridged)
     {
         mData.backup();
 
         /* first detach the current attachment */
         detach();
 
-        mData->mAttachmentType = NetworkAttachmentType_HostInterfaceNetworkAttachment;
+        mData->mAttachmentType = NetworkAttachmentType_Bridged;
 
         /* leave the lock before informing callbacks */
         alock.unlock();
@@ -833,10 +752,10 @@ STDMETHODIMP NetworkAdapter::AttachToInternalNetwork()
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     /* don't do anything if we're already internal network attached */
-    if (mData->mAttachmentType != NetworkAttachmentType_InternalNetworkAttachment)
+    if (mData->mAttachmentType != NetworkAttachmentType_Internal)
     {
         mData.backup();
 
@@ -844,15 +763,44 @@ STDMETHODIMP NetworkAdapter::AttachToInternalNetwork()
         detach();
 
         /* there must an internal network name */
-        if (   !mData->mInternalNetwork
-            || (mData->mInternalNetwork == Bstr ("")))
+        if (mData->mInternalNetwork.isEmpty())
         {
             LogRel (("Internal network name not defined, "
                      "setting to default \"intnet\"\n"));
-            mData->mInternalNetwork = Bstr ("intnet");
+            mData->mInternalNetwork = "intnet";
         }
 
-        mData->mAttachmentType = NetworkAttachmentType_InternalNetworkAttachment;
+        mData->mAttachmentType = NetworkAttachmentType_Internal;
+
+        /* leave the lock before informing callbacks */
+        alock.unlock();
+
+        mParent->onNetworkAdapterChange (this);
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP NetworkAdapter::AttachToHostOnlyInterface()
+{
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    /* the machine needs to be mutable */
+    Machine::AutoMutableStateDependency adep (mParent);
+    CheckComRCReturnRC (adep.rc());
+
+    AutoWriteLock alock (this);
+
+    /* don't do anything if we're already host interface attached */
+    if (mData->mAttachmentType != NetworkAttachmentType_HostOnly)
+    {
+        mData.backup();
+
+        /* first detach the current attachment */
+        detach();
+
+        mData->mAttachmentType = NetworkAttachmentType_HostOnly;
 
         /* leave the lock before informing callbacks */
         alock.unlock();
@@ -872,9 +820,9 @@ STDMETHODIMP NetworkAdapter::Detach()
     Machine::AutoMutableStateDependency adep (mParent);
     CheckComRCReturnRC (adep.rc());
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
-    if (mData->mAttachmentType != NetworkAttachmentType_NoNetworkAttachment)
+    if (mData->mAttachmentType != NetworkAttachmentType_Null)
     {
         mData.backup();
 
@@ -892,6 +840,238 @@ STDMETHODIMP NetworkAdapter::Detach()
 // public methods only for internal purposes
 ////////////////////////////////////////////////////////////////////////////////
 
+/**
+ *  Loads settings from the given adapter node.
+ *  May be called once right after this object creation.
+ *
+ *  @param aAdapterNode <Adapter> node.
+ *
+ *  @note Locks this object for writing.
+ */
+HRESULT NetworkAdapter::loadSettings (const settings::Key &aAdapterNode)
+{
+    using namespace settings;
+
+    AssertReturn (!aAdapterNode.isNull(), E_FAIL);
+
+    AutoCaller autoCaller (this);
+    AssertComRCReturnRC (autoCaller.rc());
+
+    AutoWriteLock alock (this);
+
+    /* Note: we assume that the default values for attributes of optional
+     * nodes are assigned in the Data::Data() constructor and don't do it
+     * here. It implies that this method may only be called after constructing
+     * a new BIOSSettings object while all its data fields are in the default
+     * values. Exceptions are fields whose creation time defaults don't match
+     * values that should be applied when these fields are not explicitly set
+     * in the settings file (for backwards compatibility reasons). This takes
+     * place when a setting of a newly created object must default to A while
+     * the same setting of an object loaded from the old settings file must
+     * default to B. */
+
+    HRESULT rc = S_OK;
+
+    /* type (optional, defaults to Am79C970A) */
+    const char *adapterType = aAdapterNode.stringValue ("type");
+
+    if (strcmp (adapterType, "Am79C970A") == 0)
+        mData->mAdapterType = NetworkAdapterType_Am79C970A;
+    else if (strcmp (adapterType, "Am79C973") == 0)
+        mData->mAdapterType = NetworkAdapterType_Am79C973;
+    else if (strcmp (adapterType, "82540EM") == 0)
+        mData->mAdapterType = NetworkAdapterType_I82540EM;
+    else if (strcmp (adapterType, "82543GC") == 0)
+        mData->mAdapterType = NetworkAdapterType_I82543GC;
+    else if (strcmp (adapterType, "82545EM") == 0)
+        mData->mAdapterType = NetworkAdapterType_I82545EM;
+    else
+        ComAssertMsgFailedRet (("Invalid adapter type '%s'", adapterType),
+                               E_FAIL);
+
+    /* enabled (required) */
+    mData->mEnabled = aAdapterNode.value <bool> ("enabled");
+    /* MAC address (can be null) */
+    rc = COMSETTER(MACAddress) (Bstr (aAdapterNode.stringValue ("MACAddress")));
+    CheckComRCReturnRC (rc);
+    /* cable (required) */
+    mData->mCableConnected = aAdapterNode.value <bool> ("cable");
+    /* line speed (defaults to 100 Mbps) */
+    mData->mLineSpeed = aAdapterNode.value <ULONG> ("speed");
+    /* tracing (defaults to false) */
+    mData->mTraceEnabled = aAdapterNode.value <bool> ("trace");
+    mData->mTraceFile = aAdapterNode.stringValue ("tracefile");
+
+    /* One of NAT, HostInerface, Internal or nothing */
+    Key attachmentNode;
+
+    if (!(attachmentNode = aAdapterNode.findKey ("NAT")).isNull())
+    {
+        /* NAT */
+
+        /* optional */
+        mData->mNATNetwork = attachmentNode.stringValue ("network");
+
+        rc = AttachToNAT();
+        CheckComRCReturnRC (rc);
+    }
+    else
+    if (!(attachmentNode = aAdapterNode.findKey ("HostInterface")).isNull() /* backwards compatibility */
+            || !(attachmentNode = aAdapterNode.findKey ("BridgedInterface")).isNull())
+    {
+        /* Host Interface Networking */
+
+        Bstr name = attachmentNode.stringValue ("name");
+        /* name can be empty, but not null */
+        ComAssertRet (!name.isNull(), E_FAIL);
+
+        rc = COMSETTER(HostInterface) (name);
+        CheckComRCReturnRC (rc);
+
+        rc = AttachToBridgedInterface();
+        CheckComRCReturnRC (rc);
+    }
+    else
+    if (!(attachmentNode = aAdapterNode.findKey ("InternalNetwork")).isNull())
+    {
+        /* Internal Networking */
+
+        /* required */
+        mData->mInternalNetwork = attachmentNode.stringValue ("name");
+        Assert (!mData->mInternalNetwork.isNull());
+
+        rc = AttachToInternalNetwork();
+        CheckComRCReturnRC (rc);
+    }
+    else
+    if (!(attachmentNode = aAdapterNode.findKey ("HostOnlyInterface")).isNull())
+    {
+#if defined(VBOX_WITH_NETFLT)
+        Bstr name = attachmentNode.stringValue ("name");
+        /* name can be empty, but not null */
+        ComAssertRet (!name.isNull(), E_FAIL);
+
+        rc = COMSETTER(HostInterface) (name);
+        CheckComRCReturnRC (rc);
+#endif
+
+        /* Host Interface Networking */
+        rc = AttachToHostOnlyInterface();
+        CheckComRCReturnRC (rc);
+    }
+    else
+    {
+        /* Adapter has no children */
+        rc = Detach();
+        CheckComRCReturnRC (rc);
+    }
+
+    return S_OK;
+}
+
+/**
+ *  Saves settings to the given adapter node.
+ *
+ *  Note that the given Adapter node is comletely empty on input.
+ *
+ *  @param aAdapterNode <Adapter> node.
+ *
+ *  @note Locks this object for reading.
+ */
+HRESULT NetworkAdapter::saveSettings (settings::Key &aAdapterNode)
+{
+    using namespace settings;
+
+    AssertReturn (!aAdapterNode.isNull(), E_FAIL);
+
+    AutoCaller autoCaller (this);
+    AssertComRCReturnRC (autoCaller.rc());
+
+    AutoReadLock alock (this);
+
+    aAdapterNode.setValue <bool> ("enabled", !!mData->mEnabled);
+    aAdapterNode.setValue <Bstr> ("MACAddress", mData->mMACAddress);
+    aAdapterNode.setValue <bool> ("cable", !!mData->mCableConnected);
+
+    aAdapterNode.setValue <ULONG> ("speed", mData->mLineSpeed);
+
+    if (mData->mTraceEnabled)
+        aAdapterNode.setValue <bool> ("trace", true);
+
+    aAdapterNode.setValueOr <Bstr> ("tracefile", mData->mTraceFile, Bstr::Null);
+
+    const char *typeStr = NULL;
+    switch (mData->mAdapterType)
+    {
+        case NetworkAdapterType_Am79C970A:
+            typeStr = "Am79C970A";
+            break;
+        case NetworkAdapterType_Am79C973:
+            typeStr = "Am79C973";
+            break;
+        case NetworkAdapterType_I82540EM:
+            typeStr = "82540EM";
+            break;
+        case NetworkAdapterType_I82543GC:
+            typeStr = "82543GC";
+            break;
+        case NetworkAdapterType_I82545EM:
+            typeStr = "82545EM";
+            break;
+        default:
+            ComAssertMsgFailedRet (("Invalid network adapter type: %d",
+                                    mData->mAdapterType),
+                                   E_FAIL);
+    }
+    aAdapterNode.setStringValue ("type", typeStr);
+
+    switch (mData->mAttachmentType)
+    {
+        case NetworkAttachmentType_Null:
+        {
+            /* do nothing -- empty content */
+            break;
+        }
+        case NetworkAttachmentType_NAT:
+        {
+            Key attachmentNode = aAdapterNode.createKey ("NAT");
+            if (!mData->mNATNetwork.isEmpty())
+                attachmentNode.setValue <Bstr> ("network",
+                                                mData->mNATNetwork);
+            break;
+        }
+        case NetworkAttachmentType_Bridged:
+        {
+            Key attachmentNode = aAdapterNode.createKey ("BridgedInterface");
+            Assert (!mData->mHostInterface.isNull());
+            attachmentNode.setValue <Bstr> ("name", mData->mHostInterface);
+            break;
+        }
+        case NetworkAttachmentType_Internal:
+        {
+            Key attachmentNode = aAdapterNode.createKey ("InternalNetwork");
+            Assert (!mData->mInternalNetwork.isEmpty());
+            attachmentNode.setValue <Bstr> ("name", mData->mInternalNetwork);
+            break;
+        }
+        case NetworkAttachmentType_HostOnly:
+        {
+            Key attachmentNode = aAdapterNode.createKey ("HostOnlyInterface");
+#if defined(VBOX_WITH_NETFLT)
+            Assert (!mData->mHostInterface.isNull());
+            attachmentNode.setValue <Bstr> ("name", mData->mHostInterface);
+#endif
+            break;
+        }
+        default:
+        {
+            ComAssertFailedRet (E_FAIL);
+        }
+    }
+
+    return S_OK;
+}
+
 /**
  *  @note Locks this object for writing.
  */
@@ -901,7 +1081,7 @@ bool NetworkAdapter::rollback()
     AutoCaller autoCaller (this);
     AssertComRCReturn (autoCaller.rc(), false);
 
-    AutoLock alock (this);
+    AutoWriteLock alock (this);
 
     bool changed = false;
 
@@ -927,11 +1107,12 @@ void NetworkAdapter::commit()
     AssertComRCReturnVoid (autoCaller.rc());
 
     /* sanity too */
-    AutoCaller thatCaller (mPeer);
-    AssertComRCReturnVoid (thatCaller.rc());
+    AutoCaller peerCaller (mPeer);
+    AssertComRCReturnVoid (peerCaller.rc());
 
-    /* lock both for writing since we modify both */
-    AutoMultiLock <2> alock (this->wlock(), AutoLock::maybeWlock (mPeer));
+    /* lock both for writing since we modify both (mPeer is "master" so locked
+     * first) */
+    AutoMultiWriteLock2 alock (mPeer, this);
 
     if (mData.isBackedUp())
     {
@@ -957,16 +1138,52 @@ void NetworkAdapter::copyFrom (NetworkAdapter *aThat)
     AssertComRCReturnVoid (autoCaller.rc());
 
     /* sanity too */
-    AutoCaller thatCaller (mPeer);
+    AutoCaller thatCaller (aThat);
     AssertComRCReturnVoid (thatCaller.rc());
 
-    /* peer is not modified, lock it for reading */
-    AutoMultiLock <2> alock (this->wlock(), aThat->rlock());
+    /* peer is not modified, lock it for reading (aThat is "master" so locked
+     * first) */
+    AutoMultiLock2 alock (aThat->rlock(), this->wlock());
 
     /* this will back up current data */
     mData.assignCopy (aThat->mData);
 }
 
+void NetworkAdapter::applyDefaults (GuestOSType *aOsType)
+{
+    AssertReturnVoid (aOsType != NULL);
+
+    /* sanity */
+    AutoCaller autoCaller (this);
+    AssertComRCReturnVoid (autoCaller.rc());
+
+    AutoWriteLock alock (this);
+
+    bool e1000enabled = false;
+#ifdef VBOX_WITH_E1000
+    e1000enabled = true;
+#endif // VBOX_WITH_E1000
+
+    NetworkAdapterType_T defaultType = aOsType->networkAdapterType();
+
+    /* Set default network adapter for this OS type */
+    if (defaultType == NetworkAdapterType_I82540EM ||
+        defaultType == NetworkAdapterType_I82543GC ||
+        defaultType == NetworkAdapterType_I82545EM)
+    {
+        if (e1000enabled) mData->mAdapterType = defaultType;
+    }
+    else mData->mAdapterType = defaultType;
+
+    /* Enable and connect the first one adapter to the NAT */
+    if (mData->mSlot == 0)
+    {
+        mData->mEnabled = true;
+        mData->mAttachmentType = NetworkAttachmentType_NAT;
+        mData->mCableConnected = true;
+    }
+}
+
 // private methods
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -977,39 +1194,41 @@ void NetworkAdapter::copyFrom (NetworkAdapter *aThat)
  */
 void NetworkAdapter::detach()
 {
-    AssertReturnVoid (isLockedOnCurrentThread());
+    AssertReturnVoid (isWriteLockOnCurrentThread());
 
     switch (mData->mAttachmentType)
     {
-        case NetworkAttachmentType_NoNetworkAttachment:
+        case NetworkAttachmentType_Null:
         {
             /* nothing to do here */
             break;
         }
-        case NetworkAttachmentType_NATNetworkAttachment:
+        case NetworkAttachmentType_NAT:
         {
             break;
         }
-        case NetworkAttachmentType_HostInterfaceNetworkAttachment:
+        case NetworkAttachmentType_Bridged:
         {
             /* reset handle and device name */
-#ifdef RT_OS_WINDOWS
             mData->mHostInterface = "";
-#endif
-#ifdef VBOX_WITH_UNIXY_TAP_NETWORKING
-            mData->mHostInterface.setNull();
-            mData->mTAPFD = NIL_RTFILE;
-#endif
             break;
         }
-        case NetworkAttachmentType_InternalNetworkAttachment:
+        case NetworkAttachmentType_Internal:
         {
             mData->mInternalNetwork.setNull();
             break;
         }
+        case NetworkAttachmentType_HostOnly:
+        {
+#if defined(VBOX_WITH_NETFLT)
+            /* reset handle and device name */
+            mData->mHostInterface = "";
+#endif
+            break;
+        }
     }
 
-    mData->mAttachmentType = NetworkAttachmentType_NoNetworkAttachment;
+    mData->mAttachmentType = NetworkAttachmentType_Null;
 }
 
 /**
@@ -1034,3 +1253,4 @@ void NetworkAdapter::generateMACAddress()
     LogFlowThisFunc (("generated MAC: '%s'\n", strMAC));
     mData->mMACAddress = strMAC;
 }
+/* vi: set tabstop=4 shiftwidth=4 expandtab: */
