@@ -1,4 +1,4 @@
-/* $Id: PerformanceImpl.cpp 18487 2009-03-29 01:35:27Z vboxsync $ */
+/* $Id: PerformanceImpl.cpp $ */
 
 /** @file
  *
@@ -488,10 +488,15 @@ void PerformanceCollector::unregisterBaseMetricsFor (const ComPtr <IUnknown> &aO
 
     AutoWriteLock alock (this);
     LogAleksey(("{%p} " LOG_FN_FMT ": before remove_if: m.baseMetrics.size()=%d\n", this, __PRETTY_FUNCTION__, m.baseMetrics.size()));
-    BaseMetricList::iterator it = std::remove_if (
-        m.baseMetrics.begin(), m.baseMetrics.end(), std::bind2nd (
-            std::mem_fun (&pm::BaseMetric::associatedWith), aObject));
-    m.baseMetrics.erase(it, m.baseMetrics.end());
+    BaseMetricList::iterator it;
+    for (it = m.baseMetrics.begin(); it != m.baseMetrics.end();)
+        if ((*it)->associatedWith(aObject))
+        {
+            delete *it;
+            m.baseMetrics.erase(it++);
+        }
+        else
+            ++it;
     LogAleksey(("{%p} " LOG_FN_FMT ": after remove_if: m.baseMetrics.size()=%d\n", this, __PRETTY_FUNCTION__, m.baseMetrics.size()));
     //LogFlowThisFuncLeave();
 }
@@ -504,10 +509,15 @@ void PerformanceCollector::unregisterMetricsFor (const ComPtr <IUnknown> &aObjec
 
     AutoWriteLock alock (this);
     LogAleksey(("{%p} " LOG_FN_FMT ": obj=%p\n", this, __PRETTY_FUNCTION__, (void *)aObject));
-    MetricList::iterator it = std::remove_if (
-        m.metrics.begin(), m.metrics.end(), std::bind2nd (
-            std::mem_fun (&pm::Metric::associatedWith), aObject));
-    m.metrics.erase(it, m.metrics.end());
+    MetricList::iterator it;
+    for (it = m.metrics.begin(); it != m.metrics.end();)
+        if ((*it)->associatedWith(aObject))
+        {
+            delete *it;
+            m.metrics.erase(it++);
+        }
+        else
+            ++it;
     //LogFlowThisFuncLeave();
 }
 
