@@ -1,4 +1,4 @@
-/* $Id: MachineImpl.h 18520 2009-03-30 08:45:50Z vboxsync $ */
+/* $Id: MachineImpl.h $ */
 
 /** @file
  *
@@ -790,6 +790,8 @@ protected:
 
     HRESULT saveStateSettings (int aFlags);
 
+    HRESULT getExtraData(const Utf8Str &aKey, Utf8Str &aValue);
+
     HRESULT createImplicitDiffs (const Bstr &aFolder,
                                  ComObjPtr <Progress> &aProgress,
                                  bool aOnline);
@@ -798,10 +800,8 @@ protected:
     void fixupHardDisks(bool aCommit, bool aOnline = false);
 
     HRESULT lockConfig();
-public: /* To have it available in VirtualBox::UnregisterMachine. */
     HRESULT unlockConfig();
 
-protected:
     /** @note This method is not thread safe */
     BOOL isConfigLocked()
     {
@@ -1116,5 +1116,6 @@ inline Machine *Machine::machine()
     return this;
 }
 
+
 #endif // ____H_MACHINEIMPL
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
