@@ -1,4 +1,4 @@
-/* $Id: dir.cpp 9358 2008-06-03 15:59:24Z vboxsync $ */
+/* $Id: dir.cpp $ */
 /** @file
  * IPRT - Directory Manipulation.
  */
@@ -42,6 +42,7 @@
 #endif
 
 #include <iprt/dir.h>
+#include <iprt/file.h>
 #include <iprt/path.h>
 #include <iprt/alloc.h>
 #include <iprt/log.h>
@@ -714,3 +715,66 @@ RTDECL(int) RTDirOpenFiltered(PRTDIR *ppDir, const char *pszPath, RTDIRFILTER en
     return rc;
 }
 
+
+RTDECL(int) RTDirRemoveRecursive(const char *pszPath)
+{
+    int rc;
+
+    if (!RTDirExists(pszPath))
+        return VINF_SUCCESS;
+
+    char szAbsPath[RTPATH_MAX];
+    /** @todo use RTPathReal here instead? */
+    rc = RTPathAbs(pszPath, szAbsPath, sizeof(szAbsPath));
+    if (RT_FAILURE(rc))
+        return rc;
+
+    PRTDIR pDir = NULL;
+    rc = RTDirOpen(&pDir, szAbsPath);
+    if (RT_SUCCESS(rc))
+    {
+        RTDIRENTRY dirEntry;
+        size_t cbDirEntry = sizeof(dirEntry);
+
+        while ((rc = RTDirRead(pDir, &dirEntry, NULL /* Passing an argument won't work here yet. */)) == VINF_SUCCESS)
+        {
+            char* pszEntry = NULL;
+            rc = RTStrAPrintf(&pszEntry, "%s/%s", szAbsPath, dirEntry.szName);
+            if(    RT_SUCCESS(rc)
+                && strcmp(dirEntry.szName, ".")
+                && strcmp(dirEntry.szName, ".."))
+            {
+                switch (dirEntry.enmType)
+                {
+                    case RTDIRENTRYTYPE_FILE:
+
+                        rc = RTFileDelete(pszEntry);
+                        break;
+
+                    case RTDIRENTRYTYPE_DIRECTORY:
+
+                        rc = RTDirRemoveRecursive(pszEntry);
+                        break;
+
+                    default:
+                        /** @todo not implemented yet. */
+                        break;
+                }
+
+                RTStrFree(pszEntry);
+            }
+
+            if (RT_FAILURE(rc))
+               break;
+        }
+        if (rc == VERR_NO_MORE_FILES)
+            rc = VINF_SUCCESS;
+
+        RTDirClose(pDir);
+        rc = RTDirRemove(szAbsPath);    
+    }
+
+    LogFlow(("RTDirRemoveRecursive(%p:{%s}): returns %Rrc\n", pszPath, pszPath, rc));
+    return rc;
+}
+
