@@ -1,4 +1,4 @@
-/* $Id: HWACCMInternal.h 18770 2009-04-06 15:00:15Z vboxsync $ */
+/* $Id: HWACCMInternal.h $ */
 /** @file
  * HWACCM - Internal header file.
  */
@@ -57,6 +57,7 @@ __BEGIN_DECLS
 /** Maximum number of exit reason statistics counters. */
 #define MAX_EXITREASON_STAT        0x100
 #define MASK_EXITREASON_STAT       0xff
+#define MASK_INJECT_IRQ_STAT       0xff
 
 /** @name Changed flags
  * These flags are used to keep track of which important registers that
@@ -643,9 +644,12 @@ typedef struct HWACCMCPU
     STAMCOUNTER             StatDRxContextSwitch;
     STAMCOUNTER             StatDRxIOCheck;
 
-
+#ifdef VBOX_WITH_STATISTICS
     R3PTRTYPE(PSTAMCOUNTER) paStatExitReason;
     R0PTRTYPE(PSTAMCOUNTER) paStatExitReasonR0;
+    R3PTRTYPE(PSTAMCOUNTER) paStatInjectedIrqs;
+    R0PTRTYPE(PSTAMCOUNTER) paStatInjectedIrqsR0;
+#endif
 } HWACCMCPU;
 /** Pointer to HWACCM VM instance data. */
 typedef HWACCMCPU *PHWACCMCPU;
