@@ -1,4 +1,4 @@
-/* $Id: tstVMStructGC.cpp 18718 2009-04-05 13:47:09Z vboxsync $ */
+/* $Id: tstVMStructGC.cpp $ */
 /** @file
  * tstVMMStructGC - Generate structure member and size checks from the GC perspective.
  *
@@ -626,7 +626,7 @@ int main()
     GEN_CHECK_OFF(PGMPOOLPAGE, GCPhys);
     GEN_CHECK_OFF(PGMPOOLPAGE, pvPageR3);
     GEN_CHECK_OFF(PGMPOOLPAGE, enmKind);
-    GEN_CHECK_OFF(PGMPOOLPAGE, bPadding);
+    GEN_CHECK_OFF(PGMPOOLPAGE, enmAccess);
     GEN_CHECK_OFF(PGMPOOLPAGE, idx);
     GEN_CHECK_OFF(PGMPOOLPAGE, iNext);
 #ifdef PGMPOOL_WITH_USER_TRACKING
